@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2023 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2023 Laszlo Molnar
+   Copyright (C) Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -28,11 +28,13 @@
 #pragma once
 
 struct Options;
-extern Options *opt;      // global options, see class PackMaster for per-file local options
 #define options_t Options // old name
 
+// global options, see class PackMaster for per-file local options
+extern Options *opt;
+
 #if WITH_THREADS
-extern std::mutex opt_lock_mutex;
+extern std::mutex opt_lock_mutex; // for locking "opt"
 #endif
 
 /*************************************************************************
@@ -54,10 +56,12 @@ enum {
 };
 
 struct Options final {
-    int cmd;
+    void reset() noexcept;
+
+    int cmd; // CMD_xxx
 
     // compression options
-    int method;
+    int method; // M_xxx
     bool method_lzma_seen;
     bool method_nrv2b_seen;
     bool method_nrv2d_seen;
@@ -65,9 +69,9 @@ struct Options final {
     int level;  // compression level 1..10
     int filter; // preferred filter from Packer::getFilters()
     bool ultra_brute;
-    bool all_methods; // try all available compression methods ?
+    bool all_methods; // try all available compression methods
     int all_methods_use_lzma;
-    bool all_filters; // try all available filters ?
+    bool all_filters; // try all available filters
     bool no_filter;   // force no filter
     bool prefer_ucl;  // prefer UCL
     bool exact;       // user requires byte-identical decompression
@@ -90,36 +94,10 @@ struct Options final {
     int verbose;
     bool to_stdout;
 
-    // debug options
-    struct {
-        int debug_level;
-        bool disable_random_id; // for Packer::getRandomId()
-        const char *dump_stub_loader;
-        char fake_stub_version[4 + 1];     // for internal debugging
-        char fake_stub_year[4 + 1];        // for internal debugging
-        bool getopt_throw_instead_of_exit; // for internal doctest checks
-        bool use_random_method;            // for internal debugging
-        bool use_random_filter;            // for internal debugging
-    } debug;
-
     // overlay handling
     enum { SKIP_OVERLAY = 0, COPY_OVERLAY = 1, STRIP_OVERLAY = 2 };
     int overlay;
 
-    // CRP - Compression Runtime Parameters (undocumented and subject to change)
-    struct {
-        lzma_compress_config_t crp_lzma;
-        ucl_compress_config_t crp_ucl;
-        zlib_compress_config_t crp_zlib;
-        zstd_compress_config_t crp_zstd;
-        void reset() noexcept {
-            crp_lzma.reset();
-            crp_ucl.reset();
-            crp_zlib.reset();
-            crp_zstd.reset();
-        }
-    } crp;
-
     // CPU options for i086/i386
     enum {
         CPU_DEFAULT = 0,
@@ -130,6 +108,34 @@ struct Options final {
     };
     int cpu_x86;
 
+    // debug options
+    struct {
+        int debug_level;
+        bool disable_random_id; // for Packer::getRandomId()
+        const char *dump_stub_loader;
+        char fake_stub_version[4 + 1];     // for internal debugging
+        char fake_stub_year[4 + 1];        // for internal debugging
+        bool getopt_throw_instead_of_exit; // for internal doctest checks
+        bool use_random_method;            // for internal debugging / fuzz testing
+        bool use_random_filter;            // for internal debugging / fuzz testing
+    } debug;
+
+    // CRP - Compression Runtime Parameters (undocumented and subject to change)
+    struct {
+        bzip2_compress_config_t crp_bzip2;
+        lzma_compress_config_t crp_lzma;
+        ucl_compress_config_t crp_ucl;
+        zlib_compress_config_t crp_zlib;
+        zstd_compress_config_t crp_zstd;
+        void reset() noexcept {
+            crp_bzip2.reset();
+            crp_lzma.reset();
+            crp_ucl.reset();
+            crp_zlib.reset();
+            crp_zstd.reset();
+        }
+    } crp;
+
     // options for various executable formats
     struct {
         bool split_segments;
@@ -154,7 +160,9 @@ struct Options final {
         unsigned char osabi0;   // replacement if 0==.e_ident[EI_OSABI]
         bool preserve_build_id; // copy the build-id to the compressed binary
         bool android_shlib;     // keep some ElfXX_Shdr for dlopen()
+        bool android_old;       // < Android_10 ==> no memfd_create, inconsistent __NR_ftruncate
         bool force_pie;         // choose DF_1_PIE instead of is_shlib
+        bool catch_sigsegv;     // to debug hardware or de-compressor
     } o_unix;
     struct {
         bool boot_only;
@@ -168,13 +176,14 @@ struct Options final {
     struct {
         int compress_exports;
         int compress_icons;
-        upx::TriBool<upx_int8_t> compress_resources;
-        upx::TriBool<upx_int8_t> compress_rt[25]; // 25 == RT_LAST
+        upx::TriBool<upx_int8_t, true> compress_resources;
+        upx::TriBool<upx_int8_t, true> compress_rt[25]; // 25 == RT_LAST
         int strip_relocs;
         const char *keep_resource;
     } win32_pe;
 
-    void reset() noexcept;
+private: // UPX conventions
+    UPX_CXX_DISABLE_NEW_DELETE(Options)
 };
 
 /* vim:set ts=4 sw=4 et: */
