@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2023 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2023 Laszlo Molnar
+   Copyright (C) Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -30,12 +30,14 @@
 #include "filter.h"
 #include "packer.h"
 #include "p_exe.h"
+#define WANT_EHDR_ENUM 1
+#include "p_elf_enum.h"
 #include "linker.h"
 
 static const CLANG_FORMAT_DUMMY_STATEMENT
 #include "stub/i086-dos16.exe.h"
 
-#define MAXMATCH 0x2000
+#define MAXMATCH     0x2000
 #define MAXRELOCSIZE (0x8000 - MAXMATCH)
 
 #define DI_LIMIT 0xff00 // see the assembly why
@@ -128,7 +130,7 @@ void PackExe::buildLoader(const Filter *) {
     exe_header_t dummy_oh;
     int flag = fillExeHeader(&dummy_oh);
 
-    initLoader(stub_i086_dos16_exe, sizeof(stub_i086_dos16_exe));
+    initLoader(EM_386, stub_i086_dos16_exe, sizeof(stub_i086_dos16_exe));
 
     if (M_IS_LZMA(ph.method)) {
         addLoader("LZMA_DEC00", opt->small ? "LZMA_DEC10" : "LZMA_DEC20", "LZMA_DEC30",
@@ -167,7 +169,7 @@ void PackExe::buildLoader(const Filter *) {
 
         info("lzma+relocator code compressed: %u -> %u", lsize, c_len_lzma);
         // reinit the loader
-        initLoader(stub_i086_dos16_exe, sizeof(stub_i086_dos16_exe));
+        initLoader(EM_386, stub_i086_dos16_exe, sizeof(stub_i086_dos16_exe));
         // prepare loader
         if (device_driver)
             addLoader("DEVICEENTRY,LZMADEVICE,DEVICEENTRY2");
@@ -229,7 +231,7 @@ int PackExe::readFileHeader() {
     return UPX_F_DOS_EXE;
 }
 
-bool PackExe::canPack() {
+tribool PackExe::canPack() {
     if (fn_has_ext(fi->getName(), "sys")) // dos/sys
         return false;
     if (!readFileHeader())
@@ -398,7 +400,7 @@ void PackExe::pack(OutputFile *fo) {
             unsigned jc = get_le32(relocs + 4 * ic);
             set_le32(relocs + 4 * ic, ((jc >> 16) * 16 + (jc & 0xffff)) & 0xfffff);
         }
-        qsort(raw_bytes(relocs, 4 * relocnum), relocnum, 4, le32_compare);
+        upx_qsort(raw_bytes(relocs, 4 * relocnum), relocnum, 4, le32_compare);
 
         SPAN_S_VAR(byte, image, ibuf + 0, ih_imagesize);
         SPAN_S_VAR(byte, crel, ibuf + ih_imagesize, ibuf);
@@ -572,7 +574,7 @@ void PackExe::pack(OutputFile *fo) {
 //
 **************************************************************************/
 
-int PackExe::canUnpack() {
+tribool PackExe::canUnpack() {
     if (!readFileHeader())
         return false;
     const unsigned off = ih.headsize16 * 16;
