@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2023 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2023 Laszlo Molnar
+   Copyright (C) Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -29,15 +29,18 @@
 #define ALLOW_INT_PLUS_MEMBUFFER 1
 #include "conf.h"
 
+#if (WITH_ZLIB)
 #include "p_elf.h"
 #include "file.h"
 #include "filter.h"
 #include "packer.h"
 #include "p_vmlinz.h"
+#define WANT_EHDR_ENUM 1
+#include "p_elf_enum.h"
 #include "linker.h"
 #include <zlib/zlib.h>
 
-static const
+static const CLANG_FORMAT_DUMMY_STATEMENT
 #include "stub/i386-linux.kernel.vmlinuz.h"
 
 static const unsigned stack_offset_during_uncompression = 0x9000;
@@ -93,7 +96,7 @@ int PackVmlinuzI386::getStrategy(Filter &/*ft*/)
 }
 
 
-bool PackVmlinuzI386::canPack()
+tribool PackVmlinuzI386::canPack()
 {
     return readFileHeader() == getFormat();
 }
@@ -267,7 +270,7 @@ int PackVmlinuzI386::decompressKernel()
             fd_pos = -1;
             // open
             fi->seek(gzoff, SEEK_SET);
-            fd = dup(fi->getFd());
+            fd = fi->dupFd();
             if (fd < 0)
                 break;
             gzFile zf = gzdopen(fd, "rb");
@@ -345,7 +348,7 @@ int PackVmlinuzI386::decompressKernel()
         // some checks
         if (fd_pos != file_size)
         {
-            NO_printf("fd_pos: %lld, file_size: %lld\n", fd_pos, file_size);
+            NO_printf("fd_pos: %jd, file_size: %lld\n", (intmax_t) fd_pos, file_size);
 
             // linux-2.6.21.5/arch/i386/boot/compressed/vmlinux.lds
             // puts .data.compressed ahead of .text, .rodata, etc;
@@ -429,7 +432,7 @@ Linker* PackVmlinuzI386::newLinker() const
 void PackVmlinuzI386::buildLoader(const Filter *ft)
 {
     // prepare loader
-    initLoader(stub_i386_linux_kernel_vmlinuz, sizeof(stub_i386_linux_kernel_vmlinuz));
+    initLoader(EM_386, stub_i386_linux_kernel_vmlinuz, sizeof(stub_i386_linux_kernel_vmlinuz));
     addLoader("LINUZ000",
               ph.first_offset_found == 1 ? "LINUZ010" : "",
               ft->id ? "LZCALLT1" : "",
@@ -504,7 +507,7 @@ void PackVmlinuzI386::pack(OutputFile *fo)
 void PackBvmlinuzI386::buildLoader(const Filter *ft)
 {
     // prepare loader
-    initLoader(stub_i386_linux_kernel_vmlinuz, sizeof(stub_i386_linux_kernel_vmlinuz));
+    initLoader(EM_386, stub_i386_linux_kernel_vmlinuz, sizeof(stub_i386_linux_kernel_vmlinuz));
     if (0!=page_offset) { // relocatable kernel
         assert(0==ft->id || 0x40==(0xf0 & ft->id));  // others assume fixed buffer address
         addLoader("LINUZ000,LINUZ001,LINUZVGA,LINUZ101,LINUZ110",
@@ -599,8 +602,12 @@ void PackBvmlinuzI386::pack(OutputFile *fo)
             properties = bswap32(properties);
         linker->defineSymbol("lzma_properties", properties);
         // -2 for properties
-        linker->defineSymbol("lzma_c_len", ph.c_len - 2);
-        linker->defineSymbol("lzma_u_len", ph.u_len);
+        if (linker->findSymbol("lzma_c_len", false)) {
+            linker->defineSymbol("lzma_c_len", ph.c_len - 2);
+        }
+        if (linker->findSymbol("lzma_u_len", false)) {
+            linker->defineSymbol("lzma_u_len", ph.u_len);
+        }
         unsigned const stack = getDecompressorWrkmemSize();
         linker->defineSymbol("lzma_stack_adjust", 0u - stack);
     }
@@ -671,7 +678,7 @@ void PackBvmlinuzI386::pack(OutputFile *fo)
 // unpack
 **************************************************************************/
 
-int PackVmlinuzI386::canUnpack()
+tribool PackVmlinuzI386::canUnpack()
 {
     if (readFileHeader() != getFormat())
         return false;
@@ -735,7 +742,7 @@ int PackVmlinuzARMEL::getStrategy(Filter &/*ft*/)
     return (opt->no_filter ? -3 : ((opt->filter > 0) ? -2 : 2));
 }
 
-bool PackVmlinuzARMEL::canPack()
+tribool PackVmlinuzARMEL::canPack()
 {
     return readFileHeader() == getFormat();
 }
@@ -849,7 +856,7 @@ int PackVmlinuzARMEL::decompressKernel()
             fd_pos = -1;
             // open
             fi->seek(gzoff, SEEK_SET);
-            fd = dup(fi->getFd());
+            fd = fi->dupFd();
             if (fd < 0)
                 break;
             gzFile zf = gzdopen(fd, "rb");
@@ -921,15 +928,15 @@ Linker* PackVmlinuzARMEL::newLinker() const
     return new ElfLinkerArmLE;
 }
 
-static const
+static const CLANG_FORMAT_DUMMY_STATEMENT
 #include "stub/arm.v5a-linux.kernel.vmlinux.h"
-static const
+static const CLANG_FORMAT_DUMMY_STATEMENT
 #include "stub/arm.v5a-linux.kernel.vmlinuz-head.h"
 
 void PackVmlinuzARMEL::buildLoader(const Filter *ft)
 {
     // prepare loader; same as vmlinux (with 'x')
-    initLoader(stub_arm_v5a_linux_kernel_vmlinux, sizeof(stub_arm_v5a_linux_kernel_vmlinux));
+    initLoader(EM_ARM, stub_arm_v5a_linux_kernel_vmlinux, sizeof(stub_arm_v5a_linux_kernel_vmlinux));
     addLoader("LINUX000", nullptr);
     if (ft->id) {
         assert(ft->calls > 0);
@@ -1029,7 +1036,7 @@ void PackVmlinuzARMEL::pack(OutputFile *fo)
         throwNotCompressible();
 }
 
-int PackVmlinuzARMEL::canUnpack()
+tribool PackVmlinuzARMEL::canUnpack()
 {
     if (readFileHeader() != getFormat())
         return false;
@@ -1067,4 +1074,6 @@ void PackVmlinuzARMEL::unpack(OutputFile *fo)
     }
 }
 
+#endif // WITH_ZLIB
+
 /* vim:set ts=4 sw=4 et: */
