@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2024 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2024 Laszlo Molnar
+   Copyright (C) Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -65,66 +65,107 @@ inline upx_rsize_t mem_size_get_n(upx_uint64_t element_size, upx_uint64_t n) may
 inline void mem_size_assert(upx_uint64_t element_size, upx_uint64_t n) may_throw {
     (void) mem_size(element_size, n); // assert size
 }
+inline void mem_size_assert_noexcept(upx_uint64_t element_size, upx_uint64_t n) noexcept {
+    assert_noexcept(mem_size_valid(element_size, n)); // assert size
+}
 
 // "new" with asserted size; will throw on invalid size
-#if DEBUG
 template <class T>
-T *NewArray(upx_uint64_t n) may_throw {
+inline T *NewT(upx_uint64_t n) may_throw {
     COMPILE_TIME_ASSERT(std::is_standard_layout<T>::value)
     COMPILE_TIME_ASSERT(std::is_trivially_copyable<T>::value)
     COMPILE_TIME_ASSERT(std::is_trivially_default_constructible<T>::value)
-    upx_rsize_t bytes = mem_size(sizeof(T), n); // assert size
+    const upx_rsize_t bytes = mem_size(sizeof(T), n); // assert size
     T *array = new T[size_t(n)];
 #if !defined(__SANITIZE_MEMORY__)
-    if (array != nullptr && bytes > 0) {
-        memset(array, 0xfb, bytes);
+    if likely (array != nullptr && bytes > 0) {
+        memset(array, 0xfb, bytes); // NOLINT(bugprone-multi-level-implicit-pointer-conversion)
         (void) VALGRIND_MAKE_MEM_UNDEFINED(array, bytes);
     }
 #endif
     UNUSED(bytes);
     return array;
 }
-#define New(type, n) (NewArray<type>((n)))
+#if DEBUG || 1
+#define New(type, n) (NewT<type>((n)))
 #else
 #define New(type, n) new type[mem_size_get_n(sizeof(type), (n))]
 #endif
 
+template <class T>
+inline T *New0T(upx_uint64_t n) may_throw {
+    COMPILE_TIME_ASSERT(std::is_standard_layout<T>::value)
+    COMPILE_TIME_ASSERT(std::is_trivially_copyable<T>::value)
+    COMPILE_TIME_ASSERT(std::is_trivially_default_constructible<T>::value)
+    const upx_rsize_t bytes = mem_size(sizeof(T), n); // assert size
+    T *array = new T[size_t(n)];
+    if likely (array != nullptr && bytes > 0)
+        memset(array, 0, bytes); // NOLINT(bugprone-multi-level-implicit-pointer-conversion)
+    return array;
+}
+#define New0(type, n) (New0T<type>((n)))
+
 /*************************************************************************
 // ptr util
 **************************************************************************/
 
+#if defined(__CHERI__) && defined(__CHERI_PURE_CAPABILITY__)
+forceinline upx_ptraddr_t ptr_get_address(const void *p) noexcept {
+    return __builtin_cheri_address_get(p);
+}
+forceinline upx_ptraddr_t ptr_get_address(upx_uintptr_t p) noexcept {
+    return __builtin_cheri_address_get(p);
+}
+#else
+forceinline upx_ptraddr_t ptr_get_address(const void *p) noexcept { return (upx_uintptr_t) p; }
+forceinline upx_ptraddr_t ptr_get_address(upx_uintptr_t p) noexcept { return p; }
+#endif
+
+forceinline upx_sptraddr_t ptraddr_diff(const void *a, const void *b) noexcept {
+    return ptr_get_address(a) - ptr_get_address(b);
+}
+
+template <size_t Alignment>
+forceinline bool ptr_is_aligned(const void *p) noexcept {
+    static_assert(upx::has_single_bit(Alignment));
+    return (ptr_get_address(p) & (Alignment - 1)) == 0;
+}
+forceinline bool ptr_is_aligned(const void *p, size_t alignment) noexcept {
+    assert_noexcept(upx::has_single_bit(alignment));
+    return (ptr_get_address(p) & (alignment - 1)) == 0;
+}
+
 // ptrdiff_t with nullptr checks and asserted size; will throw on failure
 // NOTE: returns size_in_bytes, not number of elements!
-int ptr_diff_bytes(const void *a, const void *b) may_throw;
-unsigned ptr_udiff_bytes(const void *a, const void *b) may_throw; // asserts a >= b
+noinline int ptr_diff_bytes(const void *a, const void *b) may_throw;
+noinline unsigned ptr_udiff_bytes(const void *a, const void *b) may_throw; // asserts a >= b
 
 // short names "ptr_diff" and "ptr_udiff" for types with sizeof(X) == 1
 template <class T, class U>
-inline typename std::enable_if<sizeof(T) == 1 && sizeof(U) == 1, int>::type ptr_diff(const T *a,
-                                                                                     const U *b)
-    may_throw {
+forceinline typename std::enable_if<sizeof(T) == 1 && sizeof(U) == 1, int>::type
+ptr_diff(const T *a, const U *b) may_throw {
     return ptr_diff_bytes(a, b);
 }
 template <class T, class U>
-inline typename std::enable_if<sizeof(T) == 1 && sizeof(U) == 1, unsigned>::type
+forceinline typename std::enable_if<sizeof(T) == 1 && sizeof(U) == 1, unsigned>::type
 ptr_udiff(const T *a, const U *b) may_throw {
     return ptr_udiff_bytes(a, b);
 }
 
 // check that buffers do not overlap; will throw on error
-noinline void uintptr_check_no_overlap(upx_uintptr_t a, size_t a_size, upx_uintptr_t b,
+noinline void ptraddr_check_no_overlap(upx_ptraddr_t a, size_t a_size, upx_ptraddr_t b,
                                        size_t b_size) may_throw;
-noinline void uintptr_check_no_overlap(upx_uintptr_t a, size_t a_size, upx_uintptr_t b,
-                                       size_t b_size, upx_uintptr_t c, size_t c_size) may_throw;
+noinline void ptraddr_check_no_overlap(upx_ptraddr_t a, size_t a_size, upx_ptraddr_t b,
+                                       size_t b_size, upx_ptraddr_t c, size_t c_size) may_throw;
 
 forceinline void ptr_check_no_overlap(const void *a, size_t a_size, const void *b, size_t b_size)
     may_throw {
-    uintptr_check_no_overlap((upx_uintptr_t) a, a_size, (upx_uintptr_t) b, b_size);
+    ptraddr_check_no_overlap(ptr_get_address(a), a_size, ptr_get_address(b), b_size);
 }
 forceinline void ptr_check_no_overlap(const void *a, size_t a_size, const void *b, size_t b_size,
                                       const void *c, size_t c_size) may_throw {
-    uintptr_check_no_overlap((upx_uintptr_t) a, a_size, (upx_uintptr_t) b, b_size,
-                             (upx_uintptr_t) c, c_size);
+    ptraddr_check_no_overlap(ptr_get_address(a), a_size, ptr_get_address(b), b_size,
+                             ptr_get_address(c), c_size);
 }
 
 // invalidate and poison a pointer: point to a non-null invalid address
@@ -136,16 +177,27 @@ forceinline void ptr_check_no_overlap(const void *a, size_t a_size, const void *
 //   architectures may need a more advanced/costly implementation in the future
 template <class T>
 inline void ptr_invalidate_and_poison(T *(&ptr)) noexcept {
-    ptr = (T *) (void *) 251; // 0x000000fb // NOLINT(performance-no-int-to-ptr)
+#if defined(__CHERI__) && defined(__CHERI_PURE_CAPABILITY__)
+    // TODO later: examine __builtin_cheri_bounds_set()
+    ptr = upx::ptr_static_cast<T *>(&ptr); // FIXME: HACK: point to address of self
+#else
+    // pretend there is some memory-mapped I/O at address 0x00000200
+    ptr = (T *) (void *) 512; // NOLINT(performance-no-int-to-ptr)
+#endif
 }
 
 /*************************************************************************
 // stdlib
 **************************************************************************/
 
-void *upx_calloc(size_t n, size_t element_size) may_throw;
+noinline void *upx_calloc(size_t n, size_t element_size) may_throw;
 
-void upx_memswap(void *a, void *b, size_t n) noexcept;
+noinline const char *upx_getenv(const char *envvar) noexcept;
+
+noinline void upx_memswap(void *a, void *b, size_t bytes) noexcept;
+
+noinline void upx_rand_init() noexcept;
+noinline int upx_rand() noexcept;
 
 typedef int(__acc_cdecl_qsort *upx_compare_func_t)(const void *, const void *);
 typedef void (*upx_sort_func_t)(void *array, size_t n, size_t element_size, upx_compare_func_t);
@@ -164,8 +216,8 @@ void upx_std_stable_sort(void *array, size_t n, upx_compare_func_t compare);
 #define upx_qsort(a, n, element_size, compare)                                                     \
     upx_std_stable_sort<(element_size)>((a), (n), (compare))
 #else
-// use libc qsort(); good enough for our use cases
-#define upx_qsort qsort
+                              // use libc qsort(); good enough for our use cases
+#define upx_qsort ::qsort
 #endif
 
 /*************************************************************************
@@ -192,7 +244,7 @@ bool maketempname(char *ofilename, size_t size, const char *ifilename, const cha
                   bool force = true);
 bool makebakname(char *ofilename, size_t size, const char *ifilename, bool force = true);
 
-bool is_envvar_true(const char *envvar, const char *alternate_name = nullptr) noexcept;
+noinline bool is_envvar_true(const char *envvar, const char *alternate_name = nullptr) noexcept;
 
 unsigned get_ratio(upx_uint64_t u_len, upx_uint64_t c_len);
 bool set_method_name(char *buf, size_t size, int method, int level);
